import Flutter
import UIKit
import QuickCaptureSDK 

public class QuickcapturePlugin: NSObject, FlutterPlugin {
    
    // Declare a property for the CameraHelper
    var cameraHelper: CameraHelper?
        //String array contains path of images which are compressed and captured from QC Camera
    var CapturedImages : [String] = [String]()


    public static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(name: "quickcapture", binaryMessenger: registrar.messenger())
        let instance = QuickcapturePlugin()
        
        // Initialize the CameraHelper object
        instance.cameraHelper = CameraHelper()
        
        registrar.addMethodCallDelegate(instance, channel: channel)
    }

    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
      switch call.method {
        case "initialize":
            SetConfig();
            result("Quickcapture initialised successfully")
        case "getPlatformVersion":
            result("iOS " + UIDevice.current.systemVersion)
        case "startCapture":
            if let viewController = getCurrentViewController() {
                cameraHelper?.StartCapture(sender: viewController, workingDirectory: Config.CaptureSupport.OutputPath) { (imagePaths) in
                    // On success
                    if !imagePaths.isEmpty {
                        self.CapturedImages = imagePaths
                        let jsonData = self.onActivitySuccess(imagePaths: imagePaths)
                        result(jsonData)
                    } else {
                        // On failure
                        let jsonData = self.onActivityFail(methodName: "startCapture", failedDescription: "No images captured")
                        result(jsonData)
                    }
                }
            } else {
                result(FlutterError(code: "ERROR", message: "Unable to get UIViewController", details: nil))
            }
        case "buildTiffForLastCapture":
            result(nil)
        case "buildPDFForLastCapture":
                if let pdfPath = buildPDFForLastCapture() {
                    result(pdfPath)
                } else {
                    result(FlutterError(code: "PDF_CREATION_ERROR", message: "PDF creation failed", details: nil))
                }
        case "buildPDF":
            result(nil)
        case "buildTiff":
            result(nil)
        case "compressToJPEG":
            result(nil)
        default:
            result(FlutterMethodNotImplemented)
      }
    }
      
    func getCurrentViewController() -> UIViewController? {
        guard let rootController = UIApplication.shared.keyWindow?.rootViewController else {
            return nil
        }
        var currentController: UIViewController? = rootController
        while let presentedController = currentController?.presentedViewController {
            currentController = presentedController
        }
        return currentController
    }

    class ReturnClass: Codable {
        var fileCollection: [String] = []
        var STATUS = "SUCCESS"
        var DESCRIPTION = "SUCCESS"
    }

    func onActivitySuccess(imagePaths: [String]) -> String {
        let returnClassInstance = ReturnClass()
        returnClassInstance.fileCollection = imagePaths
        
        let jsonData = try? JSONEncoder().encode(returnClassInstance)
        return String(data: jsonData!, encoding: .utf8) ?? ""
    }
    
    func onActivityFail(methodName: String, failedDescription: String?) -> String {
        var description = "Failed to capture" 
        if let failedDesc = failedDescription, !failedDesc.isEmpty {
            description += " \(failedDesc)"
        }

        let returnClassInstance = ReturnClass()
        returnClassInstance.STATUS = "FAILURE"
        returnClassInstance.DESCRIPTION = description

        let jsonData = try? JSONEncoder().encode(returnClassInstance)
        return String(data: jsonData!, encoding: .utf8) ?? ""
    }

    func buildPDFForLastCapture() -> String? {

        if self.CapturedImages.isEmpty {
            return nil
        }

        // DEV_HELP: ImgHelper class is not required to instantiate
        // we can directly use it like this
        let dpi = ImgHelper.shared.getDPI()

        // Getting Quality based on the config we set earlier
        let Quality = ImgHelper.shared.getJPEGQuality()

        guard let workingDirURL = URL(string: Config.CaptureSupport.OutputPath) else {
            return nil
        }

        // Creating temporary file name
        let fileName = String(Int(NSDate().timeIntervalSince1970))

        // Creating temporary file URL
        guard let pdfURL = URL(string: Config.CaptureSupport.OutputPath)?.appendingPathComponent(fileName + "_QCOutput_PDF.pdf") else {
            return nil
        }

        // createPdfFileFromJpegArray: This method will create a PDF out of Images provided by the NSArray of NSString saves the pdf file in outputTiffURL provided as String
        // First argument is a working directory; it should be a readable and writable directory where this function saves some temporary data then removes after completion
        // Return Integer value as 0 means TRUE or success and vice versa
        let ret = ImgHelper.shared.createPdfFileFromJpegArray(workingDirURL.path, self.CapturedImages, pdfURL.path, dpi, dpi, Quality)
        if ret == 0 {
            print("PDF Creation Success")
            return pdfURL.path
        } else {
            print("PDF Creation Failed")
            return nil
        }
    }

    func SetConfig(){
        // DEV_HELP: Some Configurations are inside ImgHelper Class
        ImgHelper.shared.setDPI(DPI._200)
        // DEV_HELP: Setting PageLayout
        ImgHelper.shared.setPageLayout(LayoutType.A4)
    }


}
