import 'package:flutter/foundation.dart';
import 'package:flutter/services.dart';

import 'quickcapture_platform_interface.dart';

/// An implementation of [QuickcapturePlatform] that uses method channels.
class MethodChannelQuickcapture extends QuickcapturePlatform {
  /// The method channel used to interact with the native platform.
  @visibleForTesting
  final methodChannel = const MethodChannel('quickcapture');

  @override
  Future<String?> initialize() async {
    String? version = await methodChannel.invokeMethod<String>('initialize');
    return version;
  }

  @override
  Future<String?> getPlatformVersion() async {
    final version =
        await methodChannel.invokeMethod<String>('getPlatformVersion');
    return version;
  }

  @override
  Future<String?> startCapture() async {
    String? result = await methodChannel.invokeMethod<String>('startCapture');
    return result;

    // methodChannel.setMethodCallHandler((call) async {
    //   if (call.method == '_onActivityResult') {
    //     int requestCode = call.arguments['requestCode'];
    //     int resultCode = call.arguments['resultCode'];
    //     dynamic data = call.arguments['data'];
    //     await _onActivityResult(requestCode, resultCode, data);
    //   }
    // });
  }

  @override
  Future<String?> buildTiffForLastCapture() async {
    String? result =
        await methodChannel.invokeMethod<String>('buildTiffForLastCapture');
    return result;
  }

  @override
  Future<String?> buildPDFForLastCapture() async {
    String? result =
        await methodChannel.invokeMethod<String>('buildPDFForLastCapture');
    return result;
  }

  @override
  Future<String?> buildPDF() async {
    String? result = await methodChannel.invokeMethod<String>('buildPDF');
    return result;
  }

  @override
  Future<String?> buildTiff() async {
    String? result = await methodChannel.invokeMethod<String>('buildTiff');
    return result;
  }

  @override
  Future<String?> compressToJPEG() async {
    String? result = await methodChannel.invokeMethod<String>('compressToJPEG');
    return result;
  }
}
