import 'package:plugin_platform_interface/plugin_platform_interface.dart';

import 'quickcapture_method_channel.dart';

abstract class QuickcapturePlatform extends PlatformInterface {
  /// Constructs a QuickcapturePlatform.
  QuickcapturePlatform() : super(token: _token);

  static final Object _token = Object();

  static QuickcapturePlatform _instance = MethodChannelQuickcapture();

  /// The default instance of [QuickcapturePlatform] to use.
  ///
  /// Defaults to [MethodChannelQuickcapture].
  static QuickcapturePlatform get instance => _instance;

  /// Platform-specific implementations should set this with their own
  /// platform-specific class that extends [QuickcapturePlatform] when
  /// they register themselves.
  static set instance(QuickcapturePlatform instance) {
    PlatformInterface.verifyToken(instance, _token);
    _instance = instance;
  }

  Future<String?> initialize() {
    throw UnimplementedError('initialize() has not been implemented.');
  }

  Future<String?> getPlatformVersion() {
    throw UnimplementedError('getPlatformVersion() has not been implemented.');
  }

  Future<String?> startCapture() {
    throw UnimplementedError('startcapture() has not been implemented.');
  }

  Future<String?> buildTiffForLastCapture() {
    throw UnimplementedError(
        'buildTiffForLastCapture() has not been implemented.');
  }

  Future<String?> buildPDFForLastCapture() {
    throw UnimplementedError(
        'buildPDFForLastCapture() has not been implemented.');
  }

  Future<String?> buildPDF() {
    throw UnimplementedError('buildPDF() has not been implemented.');
  }

  Future<String?> buildTiff() {
    throw UnimplementedError('buildTiff() has not been implemented.');
  }

  Future<String?> compressToJPEG() {
    throw UnimplementedError('compressToJPEG() has not been implemented.');
  }
}
